# Form implementation generated from reading ui file '/home/joe/Projects/imagetopixel/airunner/src/airunner/pyqt/widgets/generator_tab.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(428, 697)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        Form.setMinimumSize(QtCore.QSize(262, 0))
        Form.setMaximumSize(QtCore.QSize(16777215, 16777215))
        Form.setStyleSheet("b")
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.sectionTabWidget = QtWidgets.QTabWidget(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sectionTabWidget.sizePolicy().hasHeightForWidth())
        self.sectionTabWidget.setSizePolicy(sizePolicy)
        self.sectionTabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.sectionTabWidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.sectionTabWidget.setFont(font)
        self.sectionTabWidget.setLayoutDirection(QtCore.Qt.LayoutDirection.LeftToRight)
        self.sectionTabWidget.setAutoFillBackground(False)
        self.sectionTabWidget.setStyleSheet("")
        self.sectionTabWidget.setTabPosition(QtWidgets.QTabWidget.TabPosition.North)
        self.sectionTabWidget.setTabShape(QtWidgets.QTabWidget.TabShape.Rounded)
        self.sectionTabWidget.setElideMode(QtCore.Qt.TextElideMode.ElideNone)
        self.sectionTabWidget.setDocumentMode(False)
        self.sectionTabWidget.setTabsClosable(False)
        self.sectionTabWidget.setMovable(False)
        self.sectionTabWidget.setTabBarAutoHide(False)
        self.sectionTabWidget.setObjectName("sectionTabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.stableDiffusionTabWidget = QtWidgets.QTabWidget(parent=self.tab)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.stableDiffusionTabWidget.setFont(font)
        self.stableDiffusionTabWidget.setTabPosition(QtWidgets.QTabWidget.TabPosition.West)
        self.stableDiffusionTabWidget.setDocumentMode(False)
        self.stableDiffusionTabWidget.setMovable(False)
        self.stableDiffusionTabWidget.setObjectName("stableDiffusionTabWidget")
        self.gridLayout_4.addWidget(self.stableDiffusionTabWidget, 0, 0, 1, 1)
        self.sectionTabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setLayoutDirection(QtCore.Qt.LayoutDirection.LeftToRight)
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setSpacing(0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.kandinskyTabWidget = QtWidgets.QTabWidget(parent=self.tab_2)
        self.kandinskyTabWidget.setTabPosition(QtWidgets.QTabWidget.TabPosition.West)
        self.kandinskyTabWidget.setObjectName("kandinskyTabWidget")
        self.gridLayout_5.addWidget(self.kandinskyTabWidget, 0, 0, 1, 1)
        self.sectionTabWidget.addTab(self.tab_2, "")
        self.gridLayout.addWidget(self.sectionTabWidget, 0, 0, 1, 1)

        self.retranslateUi(Form)
        self.sectionTabWidget.setCurrentIndex(0)
        self.kandinskyTabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.sectionTabWidget.setTabText(self.sectionTabWidget.indexOf(self.tab), _translate("Form", "Stable Diffusion"))
        self.sectionTabWidget.setTabText(self.sectionTabWidget.indexOf(self.tab_2), _translate("Form", "Kandinsky"))
